/*jslint nomen: true, indent: 2, maxerr: 3 */
/*global window, rJS, Handlebars, document, loopEventListener, RSVP, URL,
   encodeURIComponent */
(function (window, rJS, jIO) {
  "use strict";
  var DEFAULT_APP_LIST_DOC = "app-list.json",
      SAMPLE_APP_LIST_DOC = "sample-app-list.json",
      BASE_CRIB_WILDCARD = ".cribjs.nexedi.net/crib-enable.html";

  function log(gadget, str, klass) {
    var p = document.createElement('p'),
      old_p_element = gadget.props.element.querySelector(klass + ' p'),
      log_element = gadget.props.element.querySelector(klass);
    if (old_p_element) {
      old_p_element.remove();
    }
    p.innerHTML = str;
    log_element.style.display = "";
    log_element.appendChild(p);
  }

  function logAddApp(gadget, str) {
    return log(gadget, str, ".crib-add-app-status");
  }

  function putNewAppList(gadget, app_list) {
    /*
    *   Add a app in the app list. As the app list is hosted directly on this
    * url, we switch back and forth with the current edit site URL in order to
    * to set it right.
    *
    */
    var current_edit_url,
        this_edit_url = new URL(
          "./crib-enable.html", window.location.href).href;
    return RSVP.Queue()
      .push(function () {
        return gadget.crib_sw_getCribEnableGadgetUrl();
      })
      .push(function (url) {
        current_edit_url = url;
        return gadget.crib_sw_setCribEnableGadgetUrl(this_edit_url);
      })
      .push(function () {
        return gadget.crib_sw_put(DEFAULT_APP_LIST_DOC, {
          content: JSON.stringify(app_list),
          type: "application/json"
        });
      })
      .push(function () {
        return gadget.crib_sw_setCribEnableGadgetUrl(current_edit_url);
      }, function () {
        return gadget.crib_sw_setCribEnableGadgetUrl(current_edit_url);
      });
  }

  function getApplicationList(gadget) {
    return RSVP.Queue()
      .push(function () {
        return gadget.fetch(DEFAULT_APP_LIST_DOC)
          .push(undefined, function () {
            return gadget.fetch(SAMPLE_APP_LIST_DOC);
          });
      })
      .push(function (data) {
        return JSON.parse(data);
      });
  }

  /*
  * This function validate the Crib URL to use, set it if correct and return
  * the scope.
  */

  function validateAndSetCribEnableGadget(gadget, crib_url, logAddApp) {
    return RSVP.Queue()
      .push(function () {
        logAddApp(gadget, "Testing Crib URL: " + crib_url);
        gadget.props.element.querySelector(".crib_enable_validator")
          .style = "display:none;";
        return gadget.getDeclaredGadget("crib_enable_validator");
      })
      .push(function (validator_gadget) {
        return validator_gadget.validateCribEnableGadgetUrl(crib_url);
      })
      .push(function (result) {
        if (!result) {
          gadget.props.element.querySelector(".crib_enable_validator")
            .style = "";
          logAddApp(gadget, "Could not Validate CribJS Gadget URL: " +
                    crib_url);
          throw "Could not Validate CribJS Gadget URL: " + crib_url;
        }
        logAddApp(gadget, "Validated Crib URL: " + crib_url);
        return gadget.crib_sw_setCribEnableGadgetUrl(crib_url);
      })
      .push(function () {
        return gadget.crib_sw_getScope();
      });
  }

  /*
  * This is the Core Function that will "install" the Application
  */

  function loadCribJSFromZip(gadget, event) {
    var app_list = [],

      // Define the list of elements for the app
      options = {
        name: gadget.props.element.querySelector(".app-name").value,
        subdomain: gadget.props.element
          .querySelector(".load-zip-to-subdomain").value,
        from_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-from-zip-path").value,
        redirect_url: gadget.props.element
          .querySelector("form.crib-load-from-zip .redirect-url").value,
        crib_url: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-to-crib-url").value,
        to_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-to-path").value
      };

    // If no Crib URL is provided construct one from the App ID
    if (!options.crib_url) {
      options.crib_url = "https://" + options.subdomain + BASE_CRIB_WILDCARD;
    }

    // Start the process to add the application
    return RSVP.Queue()
      .push(function () {
        // Check the Crib Enable Gadget and return the Scope
        return validateAndSetCribEnableGadget(gadget, options.crib_url, logAddApp);
      })
      .push(function (scope) {
        /*
        * Effectively push the App in browser
        */
        options.to_path = options.to_path ? options.to_path + "/" : options.to_path;
        options.default_page = options.to_path + options.redirect_url;
        options.redirect_to = scope + options.default_page;
        options.scope = scope;
        var promise_list = [];
        if (gadget.props.element
              .querySelector("form.crib-load-from-zip .load-zip-file").files.length !== 0) {
          logAddApp(gadget, "Will load Zip from File");
          promise_list.push(loadCribJSFromZipFile(gadget, event, logAddApp));
        } else if (gadget.props.element
            .querySelector("form.crib-load-from-zip .load-zip-url").value) {
          logAddApp(gadget, "Will load Zip from Url");
          promise_list.push(loadCribJSFromZipUrl(gadget, event, logAddApp));
        } else {
          logAddApp(gadget, "Will not load any Zip");
          promise_list.push([]);
        }
        promise_list.push(getApplicationList(gadget));
        return RSVP.all(promise_list);
      })
      .push(function (result_list) {
        /*
        * The app has been pushed, we add it to the app list
        */
        app_list = result_list[1];
        app_list.splice(
          0, 0, {
            int_index: 41,
            application_image_type: undefined,
            application_image_url: undefined,
            application_description: undefined,
            application_url: options.default_page === "index.html" ?
              options.scope : options.redirect_to,
            application_path: options.to_path,
            application_edit_page: options.redirect_to,
            application_crib: options.crib_url,
            application_title: options.name,
            application_category: undefined
          }
        );
        logAddApp(gadget, "Adding App to App list");
        return RSVP.all([
          putNewAppList(gadget, app_list),
          gadget.setSetting("editor-pattern", options.to_path)
        ]);
      })
      .push(function () {
        logAddApp(gadget, "Redirecting to Editor");
        return gadget.redirect({page: "url_list"});
      })
      .push(undefined, function (error) {
        logAddApp(gadget, error);
      });
  }

  function loadCribJSFromZipFile(gadget, event, logFunction) {
    return RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget("jio_cribjs");
    })
    .push(function (jio_cribjs_gadget) {
      return jio_cribjs_gadget.loadFromZipFile({
        path: document.location.href,
        file_list: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-file").files,
        from_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-from-zip-path").value,
        to_path: gadget.props
          .element.querySelector("form.crib-load-from-zip .load-zip-to-path").value,
        application_id: "cribjs"
      },
      function (str) {
        logFunction(gadget, str);
      });
    });
  }

  function loadCribJSFromZipUrl(gadget, event, logFunction) {
    return RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget("jio_cribjs");
    })
    .push(function (jio_cribjs_gadget) {
      return jio_cribjs_gadget.loadFromZipUrl({
        path: document.location.href,
        zip_url: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-url").value,
        from_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-from-zip-path").value,
        to_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-to-path").value,
        application_id: "cribjs"
      },
      function (str) {
        logFunction(gadget, str);
      });
    });
  }

  rJS(window)
    /////////////////////////////////////////////////////////////////
    // ready
    /////////////////////////////////////////////////////////////////

    // Assign the element to a variable
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          g.props.element = element;
          g.props.start_deferred = RSVP.defer();
        });
    })
    //////////////////////////////////////////////
    // acquired methods
    //////////////////////////////////////////////
    .declareAcquiredMethod("redirect", "redirect")
    .declareAcquiredMethod("crib_sw_get", "crib_sw_get")
    .declareAcquiredMethod("crib_sw_getScope", "crib_sw_getScope")
    .declareAcquiredMethod(
      "crib_sw_setCribEnableGadgetUrl",
      "crib_sw_setCribEnableGadgetUrl"
    )
    .allowPublicAcquisition("crib_sw_setCribEnableGadgetUrl", function (argument_list) {
      return this.crib_sw_setCribEnableGadgetUrl(argument_list[0]);
    })
    .declareAcquiredMethod(
      "crib_sw_getCribEnableGadgetUrl",
      "crib_sw_getCribEnableGadgetUrl"
    )
    .allowPublicAcquisition("crib_sw_getCribEnableGadgetUrl", function (argument_list) {
      return this.crib_sw_getCribEnableGadgetUrl();
    })
    .allowPublicAcquisition("crib_sw_get", function (argument_list) {
      return this.crib_sw_get(argument_list[0]);
    })
    .declareAcquiredMethod("crib_sw_put", "crib_sw_put")
    .allowPublicAcquisition("crib_sw_put", function (argument_list) {
      return this.crib_sw_put(argument_list[0], argument_list[1]);
    })
    .declareAcquiredMethod("setSetting", "setSetting")
    .declareMethod('render', function (options) {
      var gadget = this,
        getURL = window.location,
        site = getURL.protocol + "//" + getURL.host,
        display_details = false;
      if (options.hasOwnProperty("app_name")) {
        gadget.props.element
          .querySelector("form.crib-load-from-zip .app-name")
          .value = options.app_name;
      }
      if (options.hasOwnProperty("app_id")) {
        gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-to-subdomain")
          .value = options.app_id;
      }
      if (options.hasOwnProperty("from_path")) {
        gadget.props.element
          .querySelector("form.crib-load-from-zip .load-from-zip-path")
          .value = options.from_path;
        display_details = true;
      }
      if (options.hasOwnProperty("to_path")) {
        gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-to-path")
          .value = options.to_path;
        display_details = true;
      }
      if (options.hasOwnProperty("zip_app_url")) {
        gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-url")
          .value = options.zip_app_url;
      }
      if (options.hasOwnProperty("redirect_url")) {
        gadget.props.element
          .querySelector("form.crib-load-from-zip .redirect-url")
          .value = options.redirect_url;
        display_details = true;
      }
      if (display_details) {
        gadget.props.element
          .querySelector("details").open = true;
      }
      return gadget.props.start_deferred.resolve();
    })
    .declareMethod('fetch', function (url) {
      var context = this;
      return new RSVP.Queue()
        .push(function () {
          return jIO.util.ajax({
            url: url
          });
        })
        .push(function (evt) {
          return evt.target.responseText;
        });
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          var promise_list = [];
          // promise to get content from URL
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector("form.crib-load-from-zip"),
            'submit',
            false,
            function (event) {loadCribJSFromZip(gadget, event); }
          ));
          return RSVP.all(promise_list);
        });
    });



}(window, rJS, jIO));