/*jslint nomen: true, indent: 2, maxerr: 3 */
/*global window, rJS, Handlebars, document, loopEventListener, RSVP,
   encodeURIComponent */
(function (window, rJS, document, RSVP) {
  "use strict";

  function getParameterDict() {
    var hash = window.location.hash.substring(1),
      params = {};
    hash.split('&').map(hk => {
      let temp = hk.split('=');
      params[temp[0]] = temp[1];
    });
    return params;
  }

  function loadCribJSFromZipFile (gadget, event) {
    return RSVP.Queue()
    .push(function() {
      return gadget.getDeclaredGadget("jio_cribjs");
    })
    .push(function(jio_cribjs_gadget) {
      return jio_cribjs_gadget.loadFromZipFile({
        path: document.location.href,
        file_list: gadget.props.element
          .querySelector("form.crib-load-from-zip-file .load-zip-file").files,
        from_path: gadget.props.element
          .querySelector("form.crib-load-from-zip-file .load-from-zip-path").value,
        to_path: gadget.props
          .element.querySelector("form.crib-load-from-zip-file .load-zip-to-path").value,
        application_id: "cribjs"
      })
    })
    .push(function (url_list) {
      document.location = gadget.props.element.querySelector("form.crib-load-from-zip-file .redirect-url").value;
    })
    .push(console.log, console.log);
  }

  function loadCribJSFromZipUrl (gadget, event) {
    return RSVP.Queue()
    .push(function() {
      return gadget.getDeclaredGadget("jio_cribjs");
    })
    .push(function(jio_cribjs_gadget) {
      return jio_cribjs_gadget.loadFromZipUrl({
        path: document.location.href,
        zip_url: gadget.props.element
          .querySelector("form.crib-load-from-zip-url .load-zip-url").value,
        from_path: gadget.props.element
          .querySelector("form.crib-load-from-zip-url .load-from-zip-path").value,
        to_path: gadget.props.element
          .querySelector("form.crib-load-from-zip-url .load-zip-to-path").value,
        application_id: "cribjs"
      })
    })
    .push(function (url_list) {
      window.location.reload();
      document.location = gadget.props.element.querySelector("form.crib-load-from-zip-url .redirect-url").value;
    })
    .push(console.log, console.log);
  }

  rJS(window)
    /////////////////////////////////////////////////////////////////
    // ready
    /////////////////////////////////////////////////////////////////

    // Assign the element to a variable
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          g.props.element = element;
          g.props.start_deferred = RSVP.defer();
        });
    })
    //////////////////////////////////////////////
    // acquired methods
    //////////////////////////////////////////////
    .declareAcquiredMethod("crib_sw_get", "crib_sw_get")
    .allowPublicAcquisition("crib_sw_get", function (argument_list) {
      return this.crib_sw_get(argument_list[0]);
    })
    .declareAcquiredMethod("crib_sw_put", "crib_sw_put")
    .allowPublicAcquisition("crib_sw_put", function (argument_list) {
      return this.crib_sw_put(argument_list[0], argument_list[1]);
    })
    .declareMethod('render', function (options) {
      var gadget = this,
        getURL = window.location,
        site = getURL.protocol + "//" + getURL.host,
        params = getParameterDict();
      if ( params.hasOwnProperty("from_path") ) {
        gadget.props.element.querySelector("form.crib-load-from-zip-url .load-from-zip-path").value = params.from_path;
      } else {
        gadget.props.element.querySelector("form.crib-load-from-zip-url .load-from-zip-path").value = "";
      }
      if ( params.hasOwnProperty("to_path") ) {
        gadget.props.element.querySelector("form.crib-load-from-zip-url .load-zip-to-path").value = params.to_path;
      }
      if ( params.hasOwnProperty("zip_url") ) {
        gadget.props.element.querySelector("form.crib-load-from-zip-url .load-zip-url").value = params.zip_url;
      } else {
        gadget.props.element.querySelector("form.crib-load-from-zip-url .load-zip-url").value = "https://lab.nexedi.com/cedric.leninivin/cribjs-editor/-/archive/master/cribjs-editor-master.zip";
      }
      if ( params.hasOwnProperty("redirect_url") ) {
        gadget.props.element.querySelector("form.crib-load-from-zip-url .redirect-url").value = params.redirect_url;
      } else {
        gadget.props.element.querySelector("form.crib-load-from-zip-url .redirect-url").value = site ;
      }
      return gadget.props.start_deferred.resolve();
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          var promise_list = [];
          // promise to get content from URL
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector("form.crib-load-from-zip-url"),
            'submit',
            false,
            function (event) {loadCribJSFromZipUrl(gadget, event)}
          ));
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector("form.crib-load-from-zip-file"),
            'submit',
            false,
            function (event) {loadCribJSFromZipFile(gadget, event)}
          ));
          return RSVP.all(promise_list);
        });
    });



}(window, rJS, document, RSVP));